#ifndef VMNS_DRV_H
#define VMNS_DRV_H

#include "vmns.h"
#include "vmns_drv.h"

#define VMNS_DEV_PROMISC	0x00000001
#define VMNS_DEV_ALLMULTI	0x00000002

#ifndef __KERNEL__
struct ifreq;
#endif


	
typedef struct _vmns_skb_t {
	struct sk_buff		*pOsSkb;
	PMAC_ADDR			psrc;
	PMAC_ADDR			pdst;	
	PUSHORT				ptype;
#define					VMNS_PACKET_OTHER			1
	int					pkt_type;
	unsigned char*		ppkt_type;
	PUSHORT				pprotocol;
	PVMNS_ATTACH_INFO	pInfo;
	unsigned char*		phead;
	unsigned char*		pdata;
	unsigned char*		tcp;
	unsigned char*		ip;
	int					len;	
} VMNS_SKB, *PVMNS_SKB;

#define VMNS_GET_SKB(skb)	((skb)->pOsSkb)

typedef struct _vmns_device_stats
{
	unsigned long	rx_packets;		/* total packets received	*/
	unsigned long	tx_packets;		/* total packets transmitted	*/
	unsigned long	rx_bytes;		/* total bytes received 	*/
	unsigned long	tx_bytes;		/* total bytes transmitted	*/
	unsigned long	rx_errors;		/* bad packets received		*/
	unsigned long	tx_errors;		/* packet transmit problems	*/
	unsigned long	rx_dropped;		/* no space in linux buffers	*/
	unsigned long	tx_dropped;		/* no space available in linux	*/
	unsigned long	multicast;		/* multicast packets received	*/
	unsigned long	collisions;

	/* detailed rx_errors: */
	unsigned long	rx_length_errors;
	unsigned long	rx_over_errors;		/* receiver ring buff overflow	*/
	unsigned long	rx_crc_errors;		/* recved pkt with crc error	*/
	unsigned long	rx_frame_errors;	/* recv'd frame alignment error */
	unsigned long	rx_fifo_errors;		/* recv'r fifo overrun		*/
	unsigned long	rx_missed_errors;	/* receiver missed packet	*/

	/* detailed tx_errors */
	unsigned long	tx_aborted_errors;
	unsigned long	tx_carrier_errors;
	unsigned long	tx_fifo_errors;
	unsigned long	tx_heartbeat_errors;
	unsigned long	tx_window_errors;
	
	/* for cslip etc */
	unsigned long	rx_compressed;
	unsigned long	tx_compressed;

}	VMNS_DEVICE_STATS, *PVMNS_DEVICE_STATS;
	
typedef struct _vmns_mc_entry {
	struct _vmns_mc_entry*	next;
	MAC_ADDR				mac;
	USHORT					dmi_addrlen;
	USHORT					dmi_users;
	USHORT					dmi_gusers;
} VMNS_MC_ENTRY, *PVMNS_MC_ENTRY;

typedef struct _vmns_mc_list {
	int					num;
	PVMNS_MC_ENTRY		mc_list;
} VMNS_MC_LIST, *PVMNS_MC_LIST;

typedef struct _vmns_dev {
	struct net_device*	pOsDev;
	void*				pvoid;
	char*				name;
	PVMNS_DEVICE_STATS	stats;
	char*				dev_addr;
	MAC_ADDR			mac;
	int					(*open)			(struct _vmns_dev*);
	int					(*close)		(struct _vmns_dev*);
	int					(*send)			(struct _vmns_dev*, PVMNS_SKB);
	int					(*ioctl)		(struct _vmns_dev*, struct ifreq*,int);	
	PVMNS_DEVICE_STATS	(*get_stats)	(struct _vmns_dev*);
	int					(*set_mc_list)	(struct _vmns_dev*, PVMNS_MC_LIST,int);	
	int					(*set_mac)		(struct _vmns_dev*, void*);
	int					flags;
} VMNS_DEVICE, *PVMNS_DEVICE;

#define VMNS_GET_DEVICE(vdev)	((vdev)->pOsDev)
#define VMNS_GET_STATS(vdev)	((vdev)->stats)
extern BOOL vmns_is_24X;
extern BOOL	vmns_have_procfs;
void put_vmns_device(PVMNS_DEVICE* pvdev);
void free_vmns_device(PVMNS_DEVICE* pvdev);
PVMNS_DEVICE init_vmns_device(char* name);
int vmns_netif_rx(PVMNS_DEVICE vdev, PVMNS_SKB vskb);
int vmns_strncmp(char* str1,char* str2,int size);
char* vmns_strncpy(char* str1,char* str2,int size);
void* vmns_memcpy(void* p1,void* p2, int size);
void* vmns_memset(void* p1,int p, int size);
int vmns_memcmp(void* p1,void* p2, int size);
ULONG vmns_get_hz(void);
int	vmns_strlen(char* p);
void vmns_registerdev(PVMNS_DEVICE pvdev);
void free_vmns_mc_list(PVMNS_MC_LIST pvmc_list);
void vmns_send_packet(PVMNS_DEVICE pvdev,unsigned char* dst, unsigned char* src, 
	unsigned int type,unsigned char* pPacket,int size);
void vmns_kern_lock(void);
void vmns_kern_unlock(void);
void vmns_kern_lock_bh(void);
void vmns_kern_unlock_bh(void);
void vmns_kern_event_lock(BOOL irq,int *pflags);
void vmns_kern_event_unlock(BOOL irq,int *pflags);
void vmns_if_stop(PVMNS_DEVICE pvdev);
void vmns_if_wake(PVMNS_DEVICE pvdev);
int vmns_sprintf(char* str,const char *fmt, ...);
#endif
