/*
	Author			:	Shing
	Date			:	2001/04/22
	DESCRIPTOION	: 	define interfaces for VIA Management network service.
*/

#ifndef VMNS_H
#define VMNS_H

//#include <linux/netdevice.h>
#include "vmns_cfg.h"

//VLAN Ethernet frame type
#ifndef	ETH_P_VLAN
#define	ETH_P_VLAN	0x8100
#endif

#define IFNAMSIZ	16
#define max(a,b)	(((a)>(b)) ? (a) : (b))

#define	MAX_ADAPTER_NUM		8
#define MAX_VADAPTER_NUM	32
#define MAX_AGGREGATOR_NUM  8

#define MAX_NAMES_ENTRIES	8	//Define this constant to maxium of last contants

#define MAX_ATTRIB_SIZE		8
#ifndef VMNS_FRAME_TYPE
#define VMNS_FRAME_TYPE	0x8888
#endif
/**********************************************************************
*	VMNS constants
***********************************************************************/
#ifndef MAX_SIGNATURE_LEN
#define MAX_SIGNATURE_LEN		20
#define VMNS_SIGNATURE         	"VMNS\0"
#define VMNS_DRV_SIGNATURE		"VMNS_DRV\0"
#define VMNS_CMD_SIGNATURE		"VMNS_CMD\0"
#define VMNS_FRAME_TYPE			0x8888
#define VMNS_DEV_SIOC			(SIOCDEVPRIVATE+8)
#endif

#define VMNS_DEV_NAME		"vmns"
#ifndef IFNAMESIZ
#define IFNAMESIZ	16
#endif

/**********************************************************************
*	VMNS status
***********************************************************************/
typedef enum _VMNS_status {
	VMNS_OK=0,
	VMNS_FAIL,
	VMNS_STAM,
	VMNS_ILLEGAL_PARAM,
	VMNS_NOT_CONNECTED,
//
	VMNS_ADAPTER_NAME_EXISTS,
	VMNS_ADAPTER_NOT_FOUND,
	VMNS_ADAPTER_IN_USE,
	VMNS_ADAPTER_ATTACHED,
	VMNS_ADAPTER_OPEN_FAIL,
//
	VMNS_AGGREGATOR_NOT_FOUND,
	VMNS_AGGREGATOR_NAME_EXISTS,
	VMNS_AGGREGATOR_NO_ADAPTER,
	VMNS_AGGREGATOR_NO_VADAPTER,
	VMNS_AGGREGATOR_COMMITTED,
	VMNS_ILEGAL_OPERATION,
//	
	VMNS_VADP_OPEN_FAILED,
	VMNS_VADP_NO_VLAN,
	VMNS_VADP_TOO_MANY,
//
	VMNS_PARAM_NOT_ENOUGH_BUF,
	VMNS_IOCTL_NOT_SUPPORT,
	VMNS_STATUS_LAST
} VMNS_STATUS, *PVMN_STATUS;

/**********************************************************************
*	define indication
***********************************************************************/
typedef enum _VMNS_INDICATION {
	VMNS_XMIT_QUEUE_FULL,
	VMNS_XMIT_QUEUE_OK
} VMNS_INDICATION, *PVMNS_INDICATION;

/**********************************************************************
*	Define VIA network management op-code
***********************************************************************/
typedef enum _VMNS_OP_CODE {
	VMNS_OP_IDENTIFY=0,
	VMNS_OP_USE_ADAPTER,			
	VMNS_OP_ADD_VADAPTER,			
	VMNS_OP_ADD_AGGREGATOR,
	VMNS_OP_DEL_ADAPTER,
	VMNS_OP_DEL_VADAPTER,			//5
	VMNS_OP_DEL_AGGREGATOR,
	VMNS_OP_ATTACH_ADAPTER,
	VMNS_OP_ATTACH_VADAPTER,
	VMNS_OP_DEATTACH_ADAPTER,
	VMNS_OP_DEATTACH_VADAPTER,		//10
	VMNS_OP_QUERY_STATUS,			
	VMNS_OP_SET_AGGREGATOR_ATTRIB,
	VMNS_OP_SET_ADAPTER_ATTRIB,
	VMNS_OP_SET_VADAPTER_ATTRIB,
	VMNS_OP_SET_EVENT_HELPER,		//15
	VMNS_OP_COMMIT,
	VMNS_OP_DISCONNECT,
	VMNS_OP_LAST
} VMNS_OP_CODE, *PVMNS_OP_CODE;


typedef
struct _vmns_ioctl_cmd {
	int 			op_code;
	VMNS_STATUS		ret_status;
} VMNS_IOCTL_HDR, *PVMNS_IOCTL_HDR;

/**********************************************************************
*	Define VIA network management op-code parameter
***********************************************************************/
typedef
struct _vmns_param_identify {
	VMNS_IOCTL_HDR	hdr;
	char			Signature1[MAX_SIGNATURE_LEN];
	char			Signature2[MAX_SIGNATURE_LEN];	
	version_t		version1;
	version_t		version2;
} VMNS_PARAM_IDENTIFY, *PVMNS_PARAM_IDENTIFY;

typedef
struct _vmns_param_use_adapter {
	VMNS_IOCTL_HDR	hdr;
	char			devname[IFNAMSIZ];
} VMNS_PARAM_USE_ADAPTER, *PVMNS_PARAM_USE_ADAPTER;

typedef VMNS_PARAM_USE_ADAPTER	
	VMNS_PARAM_DEL_ADAPTER, 	*PVMNS_PARAM_DEL_ADAPTER,
	VMNS_PARAM_DEL_VADAPTER, 	*PVMNS_PARAM_DEL_VADAPTER,
	VMNS_PARAM_ADD_AGGREGATOR,	*PVMNS_PARAM_ADD_AGGREGATOR,
	VMNS_PARAM_DEL_AGGREGATOR,	*PVMNS_PARAM_DEL_AGGREGATOR;

typedef unsigned int attrib_t,		*pattrib_t;
#define VMNS_AGG_ATTRIB_NONE			0x00000000U
#define	VMNS_AGG_ATTRIB_VLAN			0x00000001U
#define VMNS_AGG_ATTRIB_VLB				0x00000002U
#define VMNS_AGG_ATTRIB_8023AD			0x00000004U
#define VMNS_AGG_ATTRIB_GLB				0x00000008U


typedef volatile unsigned int status_t,		*pstatus_t;
#define ADAP_STATUS_COMMITTED		0x00000001U
#define ADAP_STATUS_LINK_FAIL		0x00000002U
#define ADAP_STATUS_LINK_FULL		0x00000004U
#define ADAP_STATUS_SPEED_10		0x00000008U
#define ADAP_STATUS_SPEED_100		0x00000010U
#define ADAP_STATUS_SPEED_1000		0x00000020U
#define ADAP_STATUS_PRIMARY			0x00000040U
#define ADAP_STATUS_PROMISCUITY		0x00000080U

#define VADP_STATUS_COMMITTED		0x00000001U
#define VADP_STATUS_PROMISCUITY		0x00000002U

#define AGG_STATUS_COMMITTED		0x00000001U
#define AGG_STATUS_PROMISCUITY		0x00000002U
#define AGG_STATUS_VROUTING			0x00000004U

typedef	unsigned short vlan_t,		*pvlan_t;
typedef unsigned char priority_t, 	*ppriority_t;

typedef
struct _vmns_param_add_vadapter {
	VMNS_IOCTL_HDR	hdr;
	char			devname[IFNAMSIZ];
	status_t		status;
	vlan_t			vlan;
	priority_t		priority;
} VMNS_PARAM_ADD_VADAPTER, 	*PVMNS_PARAM_ADD_VADAPTER;
	
typedef
struct _vmns_param_commit {
	VMNS_IOCTL_HDR	hdr;
	char			devname[IFNAMESIZ];
} VMNS_PARAM_COMMIT, *PVMNS_PARAM_COMMIT;

typedef
struct _vmns_attach_adapter {
	VMNS_IOCTL_HDR	hdr;
	char			szAdapName[IFNAMESIZ];
	char			szAggName[IFNAMESIZ];	
} 	VMNS_PARAM_ATTACH_ADAPTER, *PVMNS_PARAM_ATTACH_ADAPTER,
	VMNS_PARAM_ATTACH_VADAPTER, *PVMNS_PARAM_ATTACH_VADAPTER,
	VMNS_PARAM_DEATTACH_ADAPTER, *PVMNS_PARAM_DEATTACH_ADAPTER,
	VMNS_PARAM_DEATTACH_VADAPTER, *PVMNS_PARAM_DEATTACH_VADAPTER;

/**************************************************************************
* Options/Attributes settings
***************************************************************************/
/*
typedef
struct _vmns_set_option {
	VMNS_IOCTL_HDR		hdr;
	char				szDevName[IFNAMESIZ];
	option_t			options;
} VMNS_PARAM_SET_VADAPTER_OPTION, *PVMNS_PARAM_SET_VADAPTER_OPTION,
  VMNS_PARAM_SET_ADAPTER_OPTION, *PVMNS_PARAM_SET_ADAPTER_OPTION,
  VMNS_PARAM_SET_AGGREGATOR_OPTION, *PVMNS_PARAM_SET_AGGREGATOR_OPTION;
*/

typedef 
enum _vmns_attrib {
	ATTRIB_VADAPTER_VLAN,
	ATTRIB_VADAPTER_PRIORITY,
	ATTRIB_AGGREGATOR_MODE,
	ATTRIB_ID_LAST
} ATTRIB_ID, *PATTRIB_ID;


typedef
struct _vmns_set_attrib {
	VMNS_IOCTL_HDR		hdr;
	char				szDevName[IFNAMESIZ];
	ATTRIB_ID			attrib_id;
	int					size;
	long				value;
} VMNS_PARAM_SET_VADAPTER_ATTRIB, *PVMNS_PARAM_SET_VADAPTER_ATTRIB,
  VMNS_PARAM_SET_ADAPTER_ATTRIB, *PVMNS_PARAM_SET_ADAPTER_ATTRIB,
  VMNS_PARAM_SET_AGGREGATOR_ATTRIB, *PVMNS_PARAM_SET_AGGREGATOR_ATTRIB;

/**************************************************************************
* Query status
***************************************************************************/
typedef
enum _vmns_query_status_id {
	QUERY_AGGREGATOR_NAMES,
	QUERY_ADAPTER_NAMES,
	QUERY_VADAPTER_NAMES,
	QUERY_AGGREGATOR_STATUS,
	QUERY_ADAPTER_STATUS,
	QUERY_VADAPTER_STATUS
} QUERY_STATUS_ID, *PQUERY_STATUS_ID;

typedef
struct _vmns_query_status_hdr {
	VMNS_IOCTL_HDR	hdr;
	QUERY_STATUS_ID	status_id;
	int				iSize;
} VMNS_PARAM_QUERY_STATUS_HEADER, *PVMNS_PARAM_QUERY_STATUS_HEADER;

typedef
struct _vmns_query_names {
	VMNS_PARAM_QUERY_STATUS_HEADER	hdr;
	int				iNum;								//Number of entries return;
	char			aszNames[MAX_NAMES_ENTRIES][IFNAMSIZ];	//the array to hold the return entries.
} VMNS_PARAM_QUERY_NAMES, *PVMNS_PARAM_QUERY_NAMES;

typedef	
struct _vmns_query_status {
	VMNS_PARAM_QUERY_STATUS_HEADER	hdr;
	char			szDevName[IFNAMSIZ];
	union {
		struct	{
			status_t		status;
			char			szAggregator[IFNAMSIZ];
//			option_t		options;
			vlan_t			vlan;
			priority_t		priority;
		}	vadp;
	
		struct {
			status_t		status;
			unsigned short	mode;
			char			aszAdapters[MAX_ADAPTER_NUM][IFNAMESIZ];
			char			aszVadapters[MAX_ADAPTER_NUM][IFNAMESIZ];
			int				iAdapters;
			int				iVadapters;
		}	agg;
		
		struct {
			status_t		status;
			char			szAggregator[IFNAMSIZ];		
		}	adp;
	} u;
}	VMNS_PARAM_QUERY_STATUS, *PVMNS_PARAM_QUERY_STATUS;

#endif
