/*
	Author	:	Shing
	Date	:	2001/04/20
*/
#undef __NO_VERSION__

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/init.h>
#include <linux/netdevice.h>
#include <linux/rtnetlink.h>
#include <linux/version.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/unaligned.h>
#include <linux/timer.h>
#ifdef LINUX_KERNEL_24X
#include <linux/spinlock.h>
#endif

#include "vmns_cfg.h"
#include "kcompat.h"
#include "vmns.h"
#include "vmns_mem.h"
#include "vmns_kernel.h"
#include "debug.h"
#include "vmns_vers.h"
#include "drv_func.h"


MODULE_AUTHOR("VIA Techologies., <ShingChuang@via.com.tw>");
MODULE_DESCRIPTION("VIA Network Management Service");
MODULE_PARM(debug,"i");
MODULE_PARM_DESC(debug,"VMNS debug level (0-3)");

MODULE_PARM(vmns_options,"i");
MODULE_PARM_DESC(vmns_options,"VMNS options (Reserved)");

int debug=0;
int vmns_options=0;

spinlock_t	vmns_lock,vmns_event_lock;

VMNS_STATUS vmns_core_init(void*);
VMNS_STATUS vmns_core_destory(void);

int vmns_command(PVMNS_IOCTL_HDR param);
int vmns_ReceiveHandler (PVMNS_SKB pvskb, char* name, struct packet_type *pt);

int kernel_version=LINUX_VERSION_CODE;

static struct net_device *vmns_device = NULL;

static int ReceiveHandler (struct sk_buff *skb, struct net_device *dev, struct packet_type *pt);

static struct packet_type vmnsPacketType =
{
		__constant_htons(VMNS_FRAME_TYPE),      // Note : no need for htons !!!
        NULL,                                   // Null = All devices 
   		ReceiveHandler, 						// Procotocol handler
        NULL,                                   // Private to the packet type 
        NULL,                                   // next
};



static int vmnsDoIoctl ( struct net_device *dev, struct ifreq *rq, 
	int cmd)
{  
	PVMNS_IOCTL_HDR params=(PVMNS_IOCTL_HDR) rq->ifr_data;	
	if (!capable(CAP_NET_ADMIN)) return -EPERM;
	switch(cmd) {
	case VMNS_DEV_SIOC:
		if (vmns_command(params)==VMNS_OK)
			return 0;
		return -EAGAIN;
		
	default:
		return -EOPNOTSUPP;
	}
	return 0;
}

 
static int  vmnsOpen( struct net_device *dev ) {
  return -EBUSY;
}

static int  vmnsStop( struct net_device *dev ) {
  return 0;
}
 
static struct net_device_stats *vmnsGetStats(struct net_device *dev ) {
	return NULL;
}

static void  vmnsSetMCList( struct net_device *dev ) {
}

static int  vmnsSend(struct sk_buff *skb, struct net_device *dev) {
	return 0;
}

static int vmns_init(void) {
        
    //init proc
			
	printk("VIA network management service Ver."SZ_VMNS_VERSION " ("__DATE__")\n");
	kern_sp_lock_init(&vmns_lock);	
	kern_sp_lock_init(&vmns_event_lock);
	if ( ( vmns_device = vmnsInitEtherdev(VMNS_DEV_NAME) ) == NULL ) {
		printk ("Init etherdev failed - cannot load the VINS module.\n");
		return -1;
	}
	
	if (vmns_device==NULL) {
		return -1;
	}
	
	vmns_device->open               =  vmnsOpen;
	vmns_device->hard_start_xmit    =  vmnsSend;
	vmns_device->stop               =  vmnsStop;
	vmns_device->get_stats          =  vmnsGetStats;
	vmns_device->set_multicast_list =  vmnsSetMCList;
	vmns_device->do_ioctl           =  vmnsDoIoctl;
	vmns_device->flags				=  0;
	
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
	vmns_device->start              =  0;
#endif

	if (vmns_core_init(vmns_device)!=VMNS_OK) {
		vmns_printk(LOG_LEVEL_NONE, "VMNS core module initailization failed.");
		return 0;
	}
	
	register_netdev(vmns_device);
	dev_add_pack ( &vmnsPacketType );	
	return 0;
}

static void vmns_cleanup(void) {
	
	dev_remove_pack( & vmnsPacketType );
	vmns_core_destory();
	unregister_netdev( vmns_device );	
	VMNS_FREE(vmns_device,sizeof(struct net_device));
}

/*Handler to receive VMNS packet*/
static int ReceiveHandler (struct sk_buff *skb, struct net_device *dev, 
						struct packet_type *pt)
{
	VMNS_SKB	vskb;
		
	if (skb->protocol==__constant_htons(VMNS_FRAME_TYPE)) {	
		vskb.pOsSkb=skb;
		vskb.psrc=(PMAC_ADDR)skb->mac.ethernet->h_dest;
		vskb.pdst=(PMAC_ADDR)skb->mac.ethernet->h_source;
		vskb.ptype=&(skb->mac.ethernet->h_proto);
		if (skb->pkt_type==PACKET_OTHERHOST)
			vskb.pkt_type=VMNS_PACKET_OTHER;
		vskb.ppkt_type=&(skb->pkt_type);
		vskb.pprotocol=&(skb->protocol);
		vskb.phead=skb->head;
		vskb.pdata=skb->data;
		vskb.len=skb->len;
		vskb.pInfo=GET_VMNS_ATTACH_INFO(skb);
		vskb.tcp=(unsigned char*) skb->h.th;
		vskb.ip=(unsigned char*) skb->nh.iph;
		if (vmns_ReceiveHandler(&vskb, dev->name, pt)!=VMNS_OK) {
			return 0;
		}		
	}
	return 0;
}

void vmns_init_timer(struct timer_list** timer) {
	*timer=VMNS_MALLOC(sizeof(struct timer_list));
	init_timer(*timer);
}

void vmns_del_timer(struct timer_list** timer) {
	if (*timer) {
		del_timer(*timer);
		VMNS_FREE(*timer,sizeof(struct timer_list));
		*timer=NULL;
	}
}

void vmns_add_timer(struct timer_list* timer) {
	add_timer(timer);
}

void vmns_set_timer(struct timer_list* ptimer, double secs, ULONG data,
		void (*function)(ULONG)){
	ptimer->expires=jiffies + ((ULONG)(secs*HZ));
	ptimer->function=function;
	ptimer->data=data;
}
          

void vmns_dev_set_allmulti(struct net_device* dev,int i) {
	dev_set_allmulti(dev,i);
}

unsigned long vmns_tick() {
	return jiffies;
}

BOOL vmns_is_dev_up(PVMNS_DEVICE pvdev) {
	struct net_device* dev=pvdev->pOsDev;

	ASSERT(dev);
	
	if (dev) {
		if (((((dev)->flags & (IFF_UP)) == (IFF_UP)) && \
                     (netif_running(dev))))
			return TRUE;
		else	
			return FALSE;
	}
	return FALSE;
}

module_init(vmns_init);
module_exit(vmns_cleanup);
