
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif

#include <linux/version.h>
#include <linux/config.h>
#include "kcompat.h"
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include "debug.h"
#include "vmns_mem.h"

struct net_device *vmnsAllocDevice( const char *name ) {       
    int allocSize;
	struct net_device *dev;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
	allocSize = sizeof(struct net_device)+16;
#else
	allocSize = sizeof(struct net_device);
#endif	
	allocSize = (allocSize + 3) & ~3;	

	dev=VMNS_MALLOC(allocSize);
	if (dev == NULL) {
	  VMNS_DBG ("kmalloc failed.\n");
	  return NULL;
	}
	
	memset( dev , 0 , allocSize );

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) 
	dev->name=(char *)(dev+1);
#endif
	
	strcpy(dev->name,name);

	return dev;
}


struct net_device *vmnsInitEtherdev( const char *name ) {
  
	struct net_device *dev;
			
	if ( (  dev = vmnsAllocDevice ( name ) ) == NULL )
		return NULL;

	ether_setup(dev); 	
	
	return dev;
}

#ifdef VMNS_DEBUG 
static int iTotalAllocated=0;
#endif

void* VMNS_MALLOC(int allocSize) {

#ifdef VMNS_DEBUG
	iTotalAllocated+=allocSize;
	printk("Total allocated %d bytes\n",iTotalAllocated);
#endif
	return kmalloc(allocSize,GFP_ATOMIC);
}

void VMNS_FREE1(const void* ptr) {
	ASSERT(ptr);
	kfree(ptr);
}

void VMNS_FREE2(const void* ptr,int size) {
#ifdef VMNS_DEBUG	
	iTotalAllocated-=size;
	ASSERT(ptr);
	printk("Total allocated %d bytes\n",iTotalAllocated);
#endif	
	if (ptr!=NULL)
		kfree(ptr);
}


void VMNS_DEV_KFREE_SKB(struct sk_buff* skb) {
	dev_kfree_skb(skb);
}

void VMNS_DEV_KFREE_SKB_IRQ(struct sk_buff* skb) {
	dev_kfree_skb_irq(skb);
}

struct sk_buff* 
VMNS_DEV_ALLOC_SKB(unsigned int length) {
	return dev_alloc_skb(length);	
}

struct sk_buff*
VMNS_DEV_SKB_CLONE(struct sk_buff* skb,int p) {
	return skb_clone(skb,p);
}
