/*
	Author	: Shing
	Date	: 2001/04/23
*/

#include <linux/version.h>

#include <linux/config.h>
#include <linux/kernel.h>

#include <linux/proc_fs.h>
#include <asm/uaccess.h>
#include <linux/skbuff.h>

#include "vmns_mem.h"
#include "debug.h"

#ifdef CONFIG_PROC_FS

#include "vmns_proc.h"


static int vmns_proc_read(char *page, char **start, off_t off, int count, int *eof,void* data);
static int vmns_proc_write(struct file *filp, const char* buffer, unsigned long count, void *data);
int vmns_proc_core_write(const char* buffer,unsigned long count,void* data);
int vmns_proc_core_read(void*, char* buf);


void* vmns_proc_mk_dir(char* name, void* parent) {
	if (parent==NULL)
		parent=proc_net;		
	return (void*) create_proc_entry(name,S_IFDIR, parent);	
}

void* vmns_proc_create_entry(char* name,void* dir,void* data) {
	struct proc_dir_entry* pentry=NULL;
	ASSERT(dir);
	
	pentry=create_proc_entry(name, S_IFREG|S_IRUSR|S_IWUSR, 
		(struct proc_dir_entry*) dir);			
		
	if (pentry) {
		pentry->read_proc= vmns_proc_read;
		pentry->write_proc= vmns_proc_write;
		pentry->data = data;
	}	
	return (void*) pentry;
}

void vmns_proc_remove_entry(const char* name, void* parent) {
	if (parent==NULL)
		parent=proc_net;
	remove_proc_entry(name,(struct proc_dir_entry *) parent);
}

static int vmns_proc_read(char *page, char **start, off_t off, int count, int *eof,void* data) {
	int len;

	len=vmns_proc_core_read(data,page);

	if (len<off+count)
		*eof=1;
	
	*start=page+off;

	len-=off;

	if (len>count)
		len=count;
	
	if (len<0)
		len=0;	
	return len;        
}

static int vmns_proc_write(struct file *filp, const char* buffer, 
		unsigned long count, void *data)
{
	int res=0;
	res=vmns_proc_core_write(buffer,count,data);
	if (res<0) {
		res=-res;
	
		switch(res) {
		case VMNS_EACCES:
				return -EACCES;
		case VMNS_EINVAL:
				return -EINVAL;
		default:
				return -EACCES;
		}
	}
	return res;
}

#endif
